#include <catch2/catch_test_macros.hpp>
#include "opencv2/core.hpp"
#include <opencv2/imgcodecs.hpp>
#include <memory>
#include <iostream>

#include "aidge/stimuli/Stimuli.hpp"
#include "aidge/backend/opencv/data/TensorImpl.hpp"
#include "aidge/data/Tensor.hpp"


using namespace Aidge;

TEST_CASE("Stimuli creation", "[Stimuli][OpenCV]") {
    SECTION("Instanciation & load an image") {
        //  Load image with imread
        cv::Mat true_mat = cv::imread("/data1/is156025/tb256203/dev/eclipse_aidge/aidge/user_tests/train-images-idx3-ubyte[00001].pgm");
        REQUIRE(true_mat.empty()==false);

        // Create Stimuli
        Stimuli stimg("/data1/is156025/tb256203/dev/eclipse_aidge/aidge/user_tests/train-images-idx3-ubyte[00001].pgm", true);
        stimg.setBackend("opencv");

        // Load the image in a tensor & save it in memory
        std::shared_ptr<Tensor> tensor_load;
        tensor_load = stimg.load();

        // Access the cv::Mat with the tensor
        TensorImpl_opencv_* tImpl_opencv = dynamic_cast<TensorImpl_opencv_*>(tensor_load->getImpl().get());

        REQUIRE(tImpl_opencv->getCvMat().size() == true_mat.size());
        REQUIRE(cv::countNonZero(tImpl_opencv->getCvMat() != true_mat) == 0);


        // This time the tensor is already loaded in memory
        std::shared_ptr<Tensor> tensor_load_2;
        tensor_load_2 = stimg.load();
        
        // Access the cv::Mat with the tensor
        TensorImpl_opencv_* tImpl_opencv_2 = dynamic_cast<TensorImpl_opencv_*>(tensor_load_2->getImpl().get());

        REQUIRE(tImpl_opencv_2->getCvMat().size() == true_mat.size());
        REQUIRE(cv::countNonZero(tImpl_opencv_2->getCvMat() != true_mat) == 0);

    }
}
